"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
/*
 * vim builtin completion items
 *
 * 1. functions
 * 2. options
 * 3. variables
 * 4. commands
 * 5. has features
 * 6. expand Keyword
 */
var fs_1 = require("fs");
var path_1 = tslib_1.__importStar(require("path"));
var fast_glob_1 = tslib_1.__importDefault(require("fast-glob"));
var util_1 = require("../util/util");
var logger_1 = tslib_1.__importDefault(require("./logger"));
var vscode_languageserver_1 = require("vscode-languageserver");
var kinds_1 = require("./kinds");
var log = logger_1.default('builtin');
var Server = /** @class */ (function () {
    function Server() {
        this.config = undefined;
        // completion items
        this.vimPredefinedVariablesItems = [];
        this.vimOptionItems = [];
        this.vimBuiltinFunctionItems = [];
        this.vimCommandItems = [];
        this.vimMapArgsItems = [];
        this.vimFeatureItems = [];
        this.expandKeywordItems = [];
        this.colorschemeItems = [];
        // signature help
        this.vimBuiltFunctionSignatureHelp = {};
        // documents
        this.vimBuiltFunctionDocuments = {};
        this.vimOptionDocuments = {};
        this.vimPredefinedVariableDocuments = {};
        this.vimCommandDocuments = {};
        this.vimFeatureDocuments = {};
        this.expandKeywordDocuments = {};
    }
    Server.prototype.init = function (config) {
        this.config = config;
        this.start();
    };
    Server.prototype.getPredefinedVimVariables = function () {
        return this.vimPredefinedVariablesItems;
    };
    Server.prototype.getVimOptions = function () {
        return this.vimOptionItems;
    };
    Server.prototype.getBuiltinVimFunctions = function () {
        return this.vimBuiltinFunctionItems;
    };
    Server.prototype.getExpandKeywords = function () {
        return this.expandKeywordItems;
    };
    Server.prototype.getVimCommands = function () {
        return this.vimCommandItems;
    };
    Server.prototype.getVimMapArgs = function () {
        return this.vimMapArgsItems;
    };
    Server.prototype.getVimFeatures = function () {
        return this.vimFeatureItems;
    };
    Server.prototype.getColorschemes = function () {
        return this.colorschemeItems;
    };
    Server.prototype.getSignatureHelpByName = function (name, idx) {
        var params = this.vimBuiltFunctionSignatureHelp[name];
        if (params) {
            return {
                signatures: [{
                        label: name + "(" + params[0] + ")" + (params[1] ? ": " + params[1] : ''),
                        documentation: this.formatVimDocument(this.vimBuiltFunctionDocuments[name], {
                            label: name,
                            kind: vscode_languageserver_1.CompletionItemKind.Function
                        }),
                        parameters: params[0].split('[')[0].split(',').map(function (param) {
                            return {
                                label: param.trim()
                            };
                        })
                    }],
                activeSignature: 0,
                activeParameter: idx
            };
        }
        return;
    };
    Server.prototype.getDocumentByCompletionItem = function (params) {
        var kind = params.kind;
        switch (kind) {
            case vscode_languageserver_1.CompletionItemKind.Variable:
                if (!this.vimPredefinedVariableDocuments[params.label]) {
                    return;
                }
                return tslib_1.__assign({}, params, { documentation: this.formatVimDocument(this.vimPredefinedVariableDocuments[params.label], params) });
            case vscode_languageserver_1.CompletionItemKind.Property:
                if (!this.vimOptionDocuments[params.label]) {
                    return;
                }
                return tslib_1.__assign({}, params, { documentation: this.formatVimDocument(this.vimOptionDocuments[params.label], params) });
            case vscode_languageserver_1.CompletionItemKind.Function:
                if (!this.vimBuiltFunctionDocuments[params.label]) {
                    return;
                }
                return tslib_1.__assign({}, params, { documentation: this.formatVimDocument(this.vimBuiltFunctionDocuments[params.label], params) });
            case vscode_languageserver_1.CompletionItemKind.EnumMember:
                if (!this.vimFeatureDocuments[params.label]) {
                    return;
                }
                return tslib_1.__assign({}, params, { documentation: this.formatVimDocument(this.vimFeatureDocuments[params.label], params) });
            case vscode_languageserver_1.CompletionItemKind.Operator:
                if (!this.vimCommandDocuments[params.label]) {
                    return;
                }
                return tslib_1.__assign({}, params, { documentation: this.formatVimDocument(this.vimCommandDocuments[params.label], params) });
            default:
                break;
        }
    };
    Server.prototype.getHoverDocument = function (name, pre, next) {
        if (util_1.isSomeMatchPattern(kinds_1.builtinVariablePattern, pre) && this.vimPredefinedVariableDocuments[name]) {
            return {
                contents: this.formatVimDocument(this.vimPredefinedVariableDocuments[name], {
                    label: name,
                    kind: vscode_languageserver_1.CompletionItemKind.Variable
                })
            };
        }
        else if (util_1.isSomeMatchPattern(kinds_1.optionPattern, pre) && this.vimOptionDocuments[name.slice(1)]) {
            return {
                contents: this.formatVimDocument(this.vimOptionDocuments[name.slice(1)], {
                    label: name,
                    kind: vscode_languageserver_1.CompletionItemKind.Property
                })
            };
        }
        else if (kinds_1.functionPattern.test(next) && this.vimBuiltFunctionDocuments[name]) {
            return {
                contents: this.formatVimDocument(this.vimBuiltFunctionDocuments[name], {
                    label: name,
                    kind: vscode_languageserver_1.CompletionItemKind.Function
                })
            };
        }
        else if (util_1.isSomeMatchPattern(kinds_1.featurePattern, pre) && this.vimFeatureDocuments[name]) {
            return {
                contents: this.formatVimDocument(this.vimFeatureDocuments[name], {
                    label: name,
                    kind: vscode_languageserver_1.CompletionItemKind.EnumMember
                })
            };
        }
        if (util_1.isSomeMatchPattern(kinds_1.commandPattern, pre) && this.vimCommandDocuments[name]) {
            return {
                contents: this.formatVimDocument(this.vimCommandDocuments[name], {
                    label: name,
                    kind: vscode_languageserver_1.CompletionItemKind.Operator
                })
            };
        }
    };
    Server.prototype.start = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var runtimepath, _a, err, docs, data;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        runtimepath = this.config.runtimepath;
                        if (runtimepath) {
                            this.resolveColorschemes(runtimepath);
                        }
                        return [4 /*yield*/, util_1.pcb(fs_1.readFile)(path_1.join(path_1.dirname(process.argv[1]), '../docs/builtin-docs.json'), 'utf-8')];
                    case 1:
                        _a = _b.sent(), err = _a[0], docs = _a[1];
                        if (err) {
                            log.error("[vimls]: read docs/builtin-doc.json fail => " + (err.message || err));
                            return [2 /*return*/];
                        }
                        try {
                            data = JSON.parse(docs);
                            this.vimBuiltinFunctionItems = data.completionItems.functions;
                            this.vimBuiltFunctionDocuments = data.documents.functions;
                            this.vimCommandItems = data.completionItems.commands;
                            this.vimCommandDocuments = data.documents.commands;
                            this.vimPredefinedVariablesItems = data.completionItems.variables;
                            this.vimPredefinedVariableDocuments = data.documents.variables;
                            this.vimOptionItems = data.completionItems.options;
                            this.vimOptionDocuments = data.documents.options;
                            this.vimFeatureItems = data.completionItems.features;
                            this.vimFeatureDocuments = data.documents.features;
                            this.expandKeywordItems = data.completionItems.expandKeywords;
                            this.expandKeywordDocuments = data.documents.expandKeywords;
                            this.vimBuiltFunctionSignatureHelp = data.signatureHelp;
                        }
                        catch (error) {
                            log.error("[vimls]: parse docs/builtin-doc.json fail => " + (error.message || error));
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    // format vim document
    Server.prototype.formatVimDocument = function (document, params) {
        var indent = 0;
        return {
            kind: vscode_languageserver_1.MarkupKind.Markdown,
            value: [
                '``` help'
            ].concat(document.map(function (line) {
                if (indent === 0) {
                    var m = line.match(/^([ \t]+)/);
                    if (m) {
                        indent = m[1].length;
                    }
                }
                return line.replace(new RegExp("^[ \\t]{" + indent + "}", 'g'), '').replace(/\t/g, '  ');
            }), [
                '```'
            ]).join('\n')
        };
    };
    Server.prototype.resolveMapArgs = function () {
        this.vimMapArgsItems = ["<buffer>", "<nowait>", "<silent>", "<script>", "<expr>", "<unique>"]
            .map(function (item) {
            return {
                label: item,
                kind: vscode_languageserver_1.CompletionItemKind.EnumMember,
                documentation: '',
                insertText: item,
                insertTextFormat: vscode_languageserver_1.InsertTextFormat.PlainText
            };
        });
    };
    Server.prototype.resolveColorschemes = function (runtimepath) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var list, glob, colorschemes;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        list = runtimepath;
                        if (this.config.vimruntime) {
                            list.push(this.config.vimruntime);
                        }
                        glob = runtimepath.map(function (p) { return path_1.default.join(p, 'colors/*.vim'); });
                        return [4 /*yield*/, fast_glob_1.default(glob, { onlyFiles: false, deep: 0 })];
                    case 1:
                        colorschemes = _a.sent();
                        this.colorschemeItems = colorschemes.map(function (p) {
                            var label = path_1.default.basename(p, '.vim');
                            var item = {
                                label: label,
                                kind: vscode_languageserver_1.CompletionItemKind.EnumMember,
                                insertText: label,
                                insertTextFormat: vscode_languageserver_1.InsertTextFormat.PlainText
                            };
                            return item;
                        });
                        return [2 /*return*/];
                }
            });
        });
    };
    return Server;
}());
exports.server = new Server();
//# sourceMappingURL=builtin.js.map